@@ -7,9 +7,9 @@ use vars qw( $VERSION $VERBOSE @EXPORT_OK);
 $VERSION = '0.01';
 $VERBOSE = 0;
 
-
 use Carp;
 
+use MBTest ();
 use Cwd ();
 use File::Basename ();
 use File::Find ();
@@ -69,39 +69,70 @@ sub chdir_all ($) {
   chdir('/') if $^O eq 'os2';
   chdir shift;
 }
+
 ########################################################################
 
+END { chdir_all(MBTest->original_cwd); }
+
 sub new {
-  my $package = shift;
+  my $self = bless {}, shift;
+  $self->reset(@_);
+}
+
+sub reset {
+  my $self = shift;
   my %options = @_;
 
   $options{name} ||= 'Simple';
-  $options{dir}  ||= Cwd::cwd();
+  $options{dir} = File::Spec->rel2abs(
+    defined $options{dir} ? $options{dir} : MBTest->tmpdir
+  );
 
   my %data = (
     no_manifest   => 0,
     xs            => 0,
     %options,
   );
-  my $self = bless( \%data, $package );
-
-  # So we can clean up later even if the caller chdir()s
-  $self->{dir} = File::Spec->rel2abs($self->{dir});
+  %$self = %data;
 
   tie %{$self->{filedata}}, 'Tie::CPHash';
 
   tie %{$self->{pending}{change}}, 'Tie::CPHash';
 
+  # start with a fresh, empty directory
   if ( -d $self->dirname ) {
     warn "Warning: Removing existing directory '@{[$self->dirname]}'\n";
-    $self->remove;
+    File::Path::rmtree( $self->dirname );
   }
+  File::Path::mkpath( $self->dirname );
 
   $self->_gen_default_filedata();
 
   return $self;
 }
 
+sub remove {
+  my $self = shift;
+  $self->chdir_original if($self->did_chdir);
+  File::Path::rmtree( $self->dirname );
+  return $self;
+}
+
+sub revert {
+  my ($self, $file) = @_;
+  if ( defined $file ) {
+    delete $self->{filedata}{$file};
+    delete $self->{pending}{$_}{$file} for qw/change remove/;
+  }
+  else {
+    delete $self->{filedata}{$_} for keys %{ $self->{filedata} };
+    for my $pend ( qw/change remove/ ) {
+      delete $self->{pending}{$pend}{$_} for keys %{ $self->{pending}{$pend} }; 
+    }
+  }
+  $self->_gen_default_filedata;
+}
+
 sub _gen_default_filedata {
   my $self = shift;
 
@@ -229,7 +260,7 @@ sub _gen_default_filedata {
 
   # 5.6 is missing const char * in its typemap
   $self->$add_unless('typemap', undent(<<"      ---"));
-      const char *              T_PV
+      const char *\tT_PV
       ---
 
   $self->$add_unless('t/basic.t', undent(<<"    ---"));
@@ -249,7 +280,6 @@ sub _gen_manifest {
   my $manifest = shift;
 
   my $fh = IO::File->new( ">$manifest" ) or do {
-    $self->remove();
     die "Can't write '$manifest'\n";
   };
 
@@ -312,7 +342,6 @@ sub regen {
       my $dirname = File::Basename::dirname( $fullname );
       unless ( -d $dirname ) {
         File::Path::mkpath( $dirname ) or do {
-          $self->remove();
           die "Can't create '$dirname'\n";
         };
       }
@@ -322,7 +351,6 @@ sub regen {
       }
 
       my $fh = IO::File->new(">$fullname") or do {
-        $self->remove();
         die "Can't write '$fullname'\n";
       };
       print $fh $self->{filedata}{$file};
@@ -339,6 +367,7 @@ sub regen {
     }
     $self->_gen_manifest( $manifest );
   }
+  return $self;
 }
 
 sub clean {
@@ -396,20 +425,7 @@ sub clean {
   }, ($^O eq 'VMS' ? './' : File::Spec->curdir) );
 
   chdir_all( $here );
-}
-
-sub remove {
-  my $self = shift;
-  croak("invalid usage -- remove()") if(@_);
-  $self->chdir_original if($self->did_chdir);
-  File::Path::rmtree( $self->dirname );
-  # might as well check
-  croak("\nthis test should have used chdir_in()") unless(Cwd::getcwd);
-}
-
-sub revert {
-  my $self = shift;
-  die "Unimplemented.\n";
+  return $self;
 }
 
 sub add_file {
@@ -425,6 +441,7 @@ sub remove_file {
   }
   delete( $self->{filedata}{$file} );
   $self->{pending}{remove}{$file} = 1;
+  return $self;
 }
 
 sub change_build_pl {
@@ -449,6 +466,7 @@ sub change_build_pl {
     );
     \$b->create_build_script();
     ---
+  return $self;
 }
 
 sub change_file {
@@ -457,6 +475,7 @@ sub change_file {
   my $data = shift;
   $self->{filedata}{$file} = $data;
   $self->{pending}{change}{$file} = 1;
+  return $self;
 }
 
 sub get_file {
@@ -468,26 +487,23 @@ sub get_file {
 
 sub chdir_in {
   my $self = shift;
-
-  $self->{original_dir} ||= Cwd::cwd; # only once
+  $self->{original_dir} ||= Cwd::cwd; # only once!
   my $dir = $self->dirname;
   chdir($dir) or die "Can't chdir to '$dir': $!";
+  return $self;
 }
 ########################################################################
 
-sub did_chdir {
-  my $self = shift;
+sub did_chdir { exists shift()->{original_dir} }
 
-  return exists($self->{original_dir});
-}
 ########################################################################
 
 sub chdir_original {
   my $self = shift;
 
-  croak("never called chdir_in()") unless($self->{original_dir});
-  my $dir = $self->{original_dir};
+  my $dir = delete $self->{original_dir};
   chdir_all($dir) or die "Can't chdir to '$dir': $!";
+  return $self;
 }
 ########################################################################
 
@@ -500,14 +516,14 @@ sub new_from_context {
 sub run_build_pl {
   my ($self, @args) = @_;
   require Module::Build;
-  Module::Build->run_perl_script('Build.PL', [], [@args])
+  return Module::Build->run_perl_script('Build.PL', [], [@args])
 }
 
 sub run_build {
   my ($self, @args) = @_;
   require Module::Build;
   my $build_script = $^O eq 'VMS' ? 'Build.com' : 'Build';
-  Module::Build->run_perl_script($build_script, [], [@args])
+  return Module::Build->run_perl_script($build_script, [], [@args])
 }
 
 1;
@@ -524,8 +540,7 @@ DistGen - Creates simple distributions for testing.
   use DistGen;
 
   # create distribution and prepare to test
-  my $dist = DistGen->new(name => 'Foo::Bar', dir => $tmp);
-  $dist->regen;
+  my $dist = DistGen->new(name => 'Foo::Bar');
   $dist->chdir_in;
 
   # change distribution files
@@ -534,42 +549,48 @@ DistGen - Creates simple distributions for testing.
   $dist->remove_file('t/some_test.t');
   $dist->regen;
 
-  # clean up extraneous files
+  # undo changes and clean up extraneous files
+  $dist->revert;
   $dist->clean;
 
   # exercise the command-line interface
   $dist->run_build_pl();
   $dist->run_build('test');
 
-  # finish testing and clean up
-  $dist->chdir_original;
-  $dist->remove;
+  # start over as a new distribution
+  $dist->reset( name => 'Foo::Bar', xs => 1 );
+  $dist->chdir_in;
 
 =head1 USAGE
 
 A DistGen object manages a set of files in a distribution directory.
 
-The constructor and some methods only define the target state of the
-distribution.  They do B<not> make any changes to the filesystem:
+The C<new()> constructor initializes the object and creates an empty
+directory for the distribution. It does not create files or chdir into
+the directory.  The C<reset()> method re-initializes the object in a
+new directory with new parameters.  It also does not create files or change
+the current directory.
+
+Some methods only define the target state of the distribution.  They do B<not>
+make any changes to the filesystem:
 
-  new
   add_file
   change_file
   change_build_pl
   remove_file
+  revert
 
 Other methods then change the filesystem to match the target state of
-the distribution (or to remove it entirely):
+the distribution:
 
-  regen
   clean
+  regen
   remove
 
 Other methods are provided for a convenience during testing. The
-most important are ones that manage the current directory:
+most important is the one to enter the distribution directory:
 
   chdir_in
-  chdir_original
 
 Additional methods portably encapsulate running Build.PL and Build:
 
@@ -578,16 +599,19 @@ Additional methods portably encapsulate running Build.PL and Build:
 
 =head1 API
 
-=head2 Constructor
+=head2 Constructors
 
 =head3 new()
 
-Create a new object.  Does not write its contents (see L</regen()>.)
+Create a new object and an empty directory to hold the distribution's files.
+If no C<dir> option is provided, it defaults to MBTest->tmpdir, which sets
+a different temp directory for Perl core testing and CPAN testing.  
+
+The C<new> method does not write any files -- see L</regen()> below.
 
-  my $tmp = MBTest->tmpdir;
   my $dist = DistGen->new(
     name        => 'Foo::Bar',
-    dir         => $tmp,
+    dir         => MBTest->tmpdir,
     xs          => 1,
     no_manifest => 0,
   );
@@ -604,9 +628,14 @@ dist name.
 
 =item dir
 
-The (parent) directory in which to create the distribution directory.
-The default is File::Spec->curdir.  The distribution will be created
-under this according to the "dist" form of C<name> (e.g. "Foo-Bar".)
+The (parent) directory in which to create the distribution directory.  The
+distribution will be created under this according to the "dist" form of C<name>
+(e.g. "Foo-Bar".)  Defaults to a temporary directory.
+
+  $dist = DistGen->new( dir => '/tmp/MB-test' );
+  $dist->regen;
+
+  # distribution files have been created in /tmp/MB-test/Simple
 
 =item xs
 
@@ -630,6 +659,13 @@ the following files are also added:
   typemap
   lib/Simple.xs # based on name parameter
 
+=head3 reset()
+
+The C<reset> method re-initializes the object as if it were generated
+from a fresh call to C<new>.  It takes the same optional parameters as C<new>.
+
+  $dist->reset( name => 'Foo::Bar', xs => 0 );
+
 =head2 Adding and editing files
 
 Note that C<$filename> should always be specified with unix-style paths,
@@ -677,6 +713,14 @@ Removes C<$filename> from the distribution.
 
   $dist->remove_file( $filename );
 
+=head3 revert()
+
+Returns the object to its initial state, or given a $filename it returns that
+file to its initial state if it is one of the built-in files.
+
+  $dist->revert;
+  $dist->revert($filename);
+
 =head2 Changing the distribution directory
 
 These methods immediately affect the filesystem.
@@ -688,8 +732,10 @@ flagged for removal with remove_file().
 
   $dist->regen(clean => 1);
 
-If the optional C<clean> argument is given, it also removes any
-extraneous files that do not belong to the distribution.
+If the optional C<clean> argument is given, it also calls C<clean>.  These
+can also be chained like this, instead:
+
+  $dist->clean->regen;
 
 =head3 clean()
 
@@ -697,22 +743,19 @@ Removes any files that are not part of the distribution.
 
   $dist->clean;
 
-=begin TODO
-
-=head3 revert()
-
-[Unimplemented] Returns the object to its initial state, or given a
-$filename it returns that file to it's initial state if it is one of
-the built-in files.
-
-  $dist->revert;
-  $dist->revert($filename);
-
-=end TODO
-
 =head3 remove()
 
-Removes the entire distribution directory.
+Changes back to the original directory and removes the distribution
+directory (but not the temporary directory set during C<new()>).  
+
+  $dist = DistGen->new->chdir->regen;
+  # ... do some testing ...
+  
+  $dist->remove->chdir_in->regen;
+  # ... do more testing ...
+
+This is like a more aggressive form of C<clean>.  Generally, calling C<clean>
+and C<regen> should be sufficient.
 
 =head2 Changing directories
 
